@@ -29,29 +29,39 @@ if hasattr(gettext, 'bind_textdomain_codeset'):
   gettext.bind_textdomain_codeset('gwibber','UTF-8')
 gettext.textdomain('gwibber')
 
-import gtk, gconf, os, json, sqlite3, uuid
-
-from . import resources
-
+import gtk, pango, gconf, os, json, uuid
 from gwibber.lib import GwibberPublic
-from gwibber.lib.gtk import *
+import util
+from microblog.util import resources
+
+for p in resources.get_plugin_dirs()[0]:
+  print "Loading plugin for %s" % p
+  globals()[p] = __import__("%s.gtk.%s" % (p,p), fromlist='*')
 
 from dbus.mainloop.glib import DBusGMainLoop
 DBusGMainLoop(set_as_default=True)
 
 class GwibberAccountManager(object):
-  def __init__(self):
+  def __init__(self, selected_account=None, condition=None, message=None):
+    self.selected_account = selected_account
+    self.condition = condition
+    self.message = message
     self.gwibber = GwibberPublic()
     self.services = json.loads(self.gwibber.GetServices())
-    
     self.ui = gtk.Builder()
     self.ui.set_translation_domain("gwibber")
     self.ui.add_from_file(resources.get_ui_asset("gwibber-accounts-dialog.ui"))
     self.ui.connect_signals(self)
 
     dialog = self.ui.get_object("accounts_dialog")
+    dialog.resize(640,400)
     dialog.set_icon_from_file(resources.get_ui_asset("gwibber.svg"))
     dialog.show_all()
+    if hasattr(gtk, "InfoBar"):
+      self.alignment_infobar = self.ui.get_object("alignment_infobar")
+      self.infobar = gtk.InfoBar()
+      self.alignment_infobar.add(self.infobar)
+      self.infobar.hide()
 
     # Hide the help button until we have help :)
     button_help = self.ui.get_object("button_help")
@@ -64,13 +74,15 @@ class GwibberAccountManager(object):
     
     self.setup_account_tree()
     self.populate_account_tree()
-    self.select_first()
+    self.select_account()
     
   def setup_account_tree(self):
     # Protocol title, Icon, Protocol Properties
     self.service_store = gtk.ListStore(str, gtk.gdk.Pixbuf, str)
     for name, props in self.services.items():
-      self.service_store.append((props["name"], self.get_icon(name), name))
+      icon = self.get_icon(name)
+      if icon:
+        self.service_store.append((props["name"], icon, name))
 
     service_combo = self.ui.get_object("protocol_combobox")
     service_combo.set_model(self.service_store)
@@ -106,14 +118,25 @@ class GwibberAccountManager(object):
     self.account_tree.append_column(name_column)
     self.account_tree.set_model(self.account_store)
 
-  def select_first(self):
-    if len(self.account_store):
-      self.account_tree.set_cursor(0)
-    else: self.add_account()
+  def select_account(self):
+    if not self.selected_account:
+      if len(self.account_store):
+        self.account_tree.set_cursor(0)
+      else: self.add_account()
+    else:
+      for i in self.account_store:
+        if i[2]["id"] == self.selected_account:
+          self.account_tree.set_cursor(i.path)
+      #self.selected_account = None
+
 
   def get_icon(self, name):
-    icf = resources.get_ui_asset("icons/breakdance/16x16/%s.png" % name)
-    return gtk.gdk.pixbuf_new_from_file(icf) if os.path.exists(icf) else None
+    icf = util.get_service_icon(name)
+    if icf:
+      if os.path.exists(icf):
+        return gtk.gdk.pixbuf_new_from_file(icf)
+    #return gtk.gdk.pixbuf_new_from_file(resources.get_ui_asset("gwibber.png"))
+    return None
 
   def get_from_keyring(self, acctid, name):
     item = {"id": str("%s/%s" % (acctid, name))}
@@ -134,17 +157,23 @@ class GwibberAccountManager(object):
     self.account_store.clear()
     accounts = json.loads(self.gwibber.GetAccounts())
     for account in accounts:
-        icon = self.get_icon(account["service"])
-        name = "%s (%s)" % (self.services[account["service"]]["name"], account["username"])
-        private_fields = [f.split(":")[-1] for f in self.services[account["service"]]["config"] if ":" in f]
-        color = None
-        for f in private_fields:
-          if not account.has_key(f): account[f] = ":KEYRING:MISSING"
-          if account[f].startswith(":KEYRING:"):
-            value = self.get_from_keyring(account["id"], f)
-            if value is None:
-              color = "pink"
-        self.account_store.append(None, [name, icon, account, color])
+        try:
+          icon = self.get_icon(account["service"])
+          if account.has_key("site_display_name"):
+            name = account["site_display_name"]
+          else:
+            name = "%s (%s)" % (self.services[account["service"]]["name"], account["username"])
+          private_fields = [f.split(":")[-1] for f in self.services[account["service"]]["config"] if ":" in f]
+          color = None
+          for f in private_fields:
+            if not account.has_key(f): account[f] = ":KEYRING:MISSING"
+            if account[f].startswith(":KEYRING:"):
+              value = self.get_from_keyring(account["id"], f)
+              if value is None:
+                color = "pink"
+          self.account_store.append(None, [name, icon, account, color])
+        except:
+          pass
 
   def verify_account(self, account):
     for field in self.services[account["service"]]["config"]:
@@ -154,29 +183,33 @@ class GwibberAccountManager(object):
     return True
 
   def on_edit_account(self, widget, data=None):
-    self.ui.get_object('vbox_save').show()
-    self.ui.get_object('vbox_create').hide()
+    self.on_edit_account_save()
+    #self.ui.get_object('vbox_save').show()
+    #self.ui.get_object('vbox_create').hide()
 
   def on_edit_account_cancel(self, widget):
-    self.ui.get_object('vbox_save').hide()
-    self.ui.get_object('vbox_create').hide()
-    self.select_first()
+    #self.ui.get_object('vbox_save').hide()
+    #self.ui.get_object('vbox_create').hide()
+    self.select_account()
 
   def on_account_delete(self, widget):
-    if "id" in self.account:
-      try:
-        private_fields = [f.split(":")[-1] for f in self.services[self.account["service"]]["config"] if ":" in f]
-        for f in private_fields:
-          if self.account[f].startswith(":KEYRING:"):
-            keyring_id = int(self.account[f].split(":")[-1])
-            gnomekeyring.item_delete_sync(gnomekeyring.get_default_keyring_sync(), keyring_id)
-      except:
-        pass  
-      self.gwibber.accounts.Delete(self.account["id"])
-    self.populate_account_tree()
-    self.select_first()
+    if not hasattr(self, 'account'):
+      pass
+    else:  
+      if "id" in self.account:
+        try:
+          private_fields = [f.split(":")[-1] for f in self.services[self.account["service"]]["config"] if ":" in f]
+          for f in private_fields:
+            if self.account[f].startswith(":KEYRING:"):
+              keyring_id = int(self.account[f].split(":")[-1])
+              gnomekeyring.item_delete_sync(gnomekeyring.get_default_keyring_sync(), keyring_id)
+        except:
+          pass  
+        self.gwibber.accounts.Delete(self.account["id"])
+      self.populate_account_tree()
+      self.select_account()
 
-  def on_edit_account_save(self, widget):
+  def on_edit_account_save(self, widget=None):
     print "Saving..."
     
     service = self.services[self.account["service"]]
@@ -214,6 +247,13 @@ class GwibberAccountManager(object):
 
     self.ui.get_object('vbox_save').hide()
     self.ui.get_object('vbox_create').hide()
+
+    # if we just fixed a failure case, hide the infobar
+    if self.account["id"] == self.selected_account:
+      if hasattr(gtk, "InfoBar"):
+        self.infobar.hide()
+      self.selected_account = None
+
     self.populate_account_tree()
 
     # Set the autostart gconf key so we get loaded on login
@@ -241,7 +281,24 @@ class GwibberAccountManager(object):
     service = model.get_value(iter, 2)
     self.account_show(service, icon)
 
-  def account_show(self, service_id, icon=None, account=None):
+  def account_show(self, service_id, icon=None, account=None, condition=None, message=None):
+    if hasattr(gtk, "InfoBar"):
+      if condition == "error":
+        self.infobar.set_message_type(gtk.MESSAGE_ERROR)
+      if condition == "info":
+        self.infobar.set_message_type(gtk.MESSAGE_INFO)
+      if message and condition:
+        if hasattr(self, "infobar_content_area"):
+          for child in self.infobar_content_area.get_children(): child.destroy()
+        self.infobar_content_area = self.infobar.get_content_area()
+        message_label = gtk.Label(message)
+        message_label.set_use_markup(True)
+        message_label.set_ellipsize(pango.ELLIPSIZE_END)
+        self.infobar_content_area.add(message_label)
+        self.infobar.show_all()
+      else:
+        self.infobar.hide()
+
     vbox_account = self.ui.get_object('vbox_account')
     for child in vbox_account.get_children(): child.destroy()
 
@@ -262,7 +319,7 @@ class GwibberAccountManager(object):
       else:
         is_new = False
 
-      aw = globals()[service_id].AccountWidget(account, self.ui)
+      aw = globals()[service_id].AccountWidget(account, self)
 
       vbox_account.pack_start(aw, False, False)
       vbox_account.show()
@@ -313,7 +370,10 @@ class GwibberAccountManager(object):
       iter = model.get_iter(rows[0][0])
       icon = model.get_value(iter, 1)
       acct = model.get_value(iter, 2)
-      self.account_show(acct["service"], icon, acct)
+      if self.selected_account == acct["id"] and self.condition:
+        self.account_show(acct["service"], icon, acct, self.condition, self.message)
+      else:
+        self.account_show(acct["service"], icon, acct)
 
   def on_button_close_clicked(self, widget, data=None):
     gtk.main_quit()
