@@ -1,4 +1,4 @@
-import gtk, gwui, microblog, resources, util, json
+import gtk, gwui, microblog, util, json
 from microblog.util.const import *
 # Try to import * from custom, install custom.py to include packaging 
 # customizations like distro API keys, etc
@@ -8,6 +8,7 @@ except:
   pass
 
 import gwibber.microblog.util
+from gwibber.microblog.util import resources
 import mx.DateTime
 import re
 
@@ -19,7 +20,7 @@ class MessageAction:
 
   @classmethod
   def get_icon_path(self, size=16, use_theme=True):
-    return resources.icon(self.icon, size, use_theme)
+    return util.icon(self.icon, size, use_theme)
     
   @classmethod
   def include(self, client, msg):
