@@ -1,9 +1,10 @@
 #!/usr/bin/env python
 
-import gtk, gobject, gwui, util, resources, actions, json, gconf
-import subprocess, os, sqlite3, time, datetime
+import gtk, gobject, gwui, util, actions, json, gconf
+import subprocess, os, time, datetime
 from microblog import config
 import microblog.util
+from microblog.util import resources
 
 import gettext
 from gettext import lgettext as _
@@ -20,6 +21,11 @@ try:
 except:
   pass
 
+try:
+  import appindicator
+except:
+  appindicator = None
+
 from dbus.mainloop.glib import DBusGMainLoop
 import dbus, dbus.service
 
@@ -34,16 +40,20 @@ class GwibberClient(gtk.Window):
     self.model = gwui.Model()
 
     self.service = self.model.daemon
+    self.messages = self.model.messages
     self.service.connect_to_signal("LoadingStarted", self.on_loading_started)
     self.service.connect_to_signal("LoadingComplete", self.on_loading_complete)
     self.service.connect_to_signal("IndicatorInterestAdded", self.on_indicator_interest_added)
     self.service.connect_to_signal("IndicatorInterestRemoved", self.on_indicator_interest_removed)
 
+    self.messages.connect_to_signal("Message", self.on_new_message)
+
     self.connection = microblog.util.getbus("Connection")
     self.connection.connect_to_signal("ConnectionOnline", self.on_connection_online)
     self.connection.connect_to_signal("ConnectionOffline", self.on_connection_offline)
 
     self.accounts = microblog.util.getbus("Accounts")
+    self.quicklist_messages = {}
 
     first_run = False
     # check for existing accounts
@@ -88,6 +98,7 @@ class GwibberClient(gtk.Window):
 
     self.set_property("skip-taskbar-hint", self.model.settings["hide_taskbar_entry"])
 
+
     # set state online/offline
     if not self.connection.isConnected():
       log.logger.info("Setting to Offline")
@@ -108,6 +119,29 @@ class GwibberClient(gtk.Window):
   def on_configure_event(self, *args):
     self.save_window_settings()
 
+  def on_new_message(self, change,  message):
+    message = json.loads(message)
+    if appindicator:
+      if message["stream"] == "messages":
+        if not message["sender"]["is_me"]:
+          if not message["sender"].has_key("name"): return
+          if self.quicklist_messages.has_key(message["sender"]["name"]):
+            count = self.quicklist_messages[message["sender"]["name"]]["count"] + 1
+            msg = "%s (%s)" % (message["sender"]["name"], count)
+            self.quicklist_messages[message["sender"]["name"]]["count"] = count
+            self.quicklist_messages[message["sender"]["name"]]["menu"].set_label(msg)
+            self.quicklist_messages[message["sender"]["name"]]["menu"].show()
+          else:
+            msg = "%s" % message["sender"]["name"]
+            ai_entry = gtk.MenuItem(msg)
+            ai_entry.connect("activate", lambda *a: self.present_with_time(int(time.mktime(datetime.datetime.now().timetuple()))))
+            self.quicklist_messages[message["sender"]["name"]] = {"count": 1, "menu": ai_entry}
+            self.ai_menu.prepend(ai_entry)
+            ai_entry.show()
+          while len(self.quicklist_messages) >= 7:
+            self.quicklist_messages.values()[-1]["menu"].destroy()
+            self.quicklist_messages.pop(self.quicklist_messages.keys()[-1])
+
   def on_indicator_interest_added(self):
     # Hide the notification area icon if there is interest from the 
     # messaging menu indicator
@@ -177,6 +211,7 @@ class GwibberClient(gtk.Window):
     self.input = gwui.Input()
     self.input.connect("submit", self.on_input_activate)
     self.input.connect("changed", self.on_input_changed)
+    self.input.connect("clear", self.on_input_cleared)
 
     self.input_splitter = gtk.VPaned()
     self.input_splitter.pack1(self.stream_view, resize=True)
@@ -215,6 +250,10 @@ class GwibberClient(gtk.Window):
 
     self.stream_view.set_state(streams)
     self.update_menu_availability()
+    for stream in streams:
+      if stream["stream"]:
+        gobject.idle_add(lambda: self.service.UpdateIndicators(stream["stream"]))
+
 
   def on_toggle_window_visibility(self, w):
     if self.get_property("visible"):
@@ -315,6 +354,35 @@ class GwibberClient(gtk.Window):
     key, mod = gtk.accelerator_parse("<ctrl>R")
     refresh.add_accelerator("activate", ui.get_accel_group(), key, mod, gtk.ACCEL_VISIBLE)
 
+    if appindicator:
+      # Use appindicators to get quicklists in unity
+      self.ind = appindicator.Indicator ("Gwibber",
+                                    "applications-microblogging-panel",
+                                    appindicator.CATEGORY_APPLICATION_STATUS)
+
+      # create a menu
+      self.ai_menu = gtk.Menu()
+
+      self.ai_menu.append(gtk.SeparatorMenuItem())
+      ai_refresh = gtk.MenuItem(_("_Refresh"))
+      ai_refresh.connect("activate", self.on_refresh)
+      self.ai_menu.append(ai_refresh)
+
+      ai_accounts = gtk.MenuItem(_("_Accounts"))
+      ai_accounts.connect("activate", self.on_accounts)
+      self.ai_menu.append(ai_accounts)
+
+      ai_preferences = gtk.MenuItem(_("_Preferences"))
+      ai_preferences.connect("activate", self.on_preferences)
+      self.ai_menu.append(ai_preferences)
+
+      # show the items
+      self.ai_menu.show_all()
+
+      self.ind.set_menu(self.ai_menu)
+
+      # End use appindicators to get quicklists in unity
+
     return ui
 
   def update_menu_availability(self):
@@ -352,9 +420,8 @@ class GwibberClient(gtk.Window):
     self.input.clear()
 
   def get_message(self, id):
-    with sqlite3.connect(SQLITE_DB_FILENAME) as db:
-      data = db.execute("SELECT data FROM messages WHERE id=?", (id,)).fetchone()
-    if data: return json.loads(data[0])
+    data = self.messages.Get(id)
+    if data: return json.loads(data)
 
   def on_refresh(self, *args):
     self.service.Refresh()
@@ -482,10 +549,16 @@ class GwibberClient(gtk.Window):
 
   def on_stream_changed(self, widget, stream):
     self.update_menu_availability()
+    if stream["stream"]:
+      gobject.idle_add(lambda: self.service.UpdateIndicators(stream["stream"]))
 
   def on_input_changed(self, w, text, cnt):
     self.input.textview.set_overlay_text(str(MAX_MESSAGE_LENGTH - cnt))
 
+  def on_input_cleared(self, seq):
+      self.message_target.end()
+      self.input.clear()
+
   def on_input_activate(self, w, text, cnt):
     self.send_message(text)
     self.input.clear()
@@ -495,6 +568,8 @@ class GwibberClient(gtk.Window):
     self.input.clear()
 
   def send_message(self, text):
+    if len(text) == 0:
+      return
     target = self.message_target.target
     action = self.message_target.action
 
@@ -544,7 +619,7 @@ class Client(dbus.service.Object):
     dbus.service.Object.__init__(self, self.bus_name, self.__dbus_object_path__)
 
     # Methods the client exposes via dbus, return from the list method
-    self.exposed_methods = ["focus_client", "show_replies"]
+    self.exposed_methods = ["focus_client", "show_stream"]
     self.w = GwibberClient()
 
   @dbus.service.method("com.GwibberClient", in_signature="", out_signature="")
@@ -559,15 +634,15 @@ class Client(dbus.service.Object):
     except:
       pass
 
-  @dbus.service.method("com.GwibberClient", in_signature="", out_signature="")
-  def show_replies(self):
+  @dbus.service.method("com.GwibberClient", in_signature="s", out_signature="")
+  def show_stream(self, stream):
     """
     This method focuses the client UI and displays the replies view.
     Currently used when activated via the messaging indicator.
     """
     self.w.present_with_time(int(time.mktime(datetime.datetime.now().timetuple())))
     self.w.move(*self.w.model.settings["window_position"])
-    stream = {'account': None, 'stream': 'replies', 'transient': False}
+    stream = {'account': None, 'stream': stream, 'transient': False}
     if isinstance(self.w.stream_view, gwui.MultiStreamUi):
       streams = self.w.stream_view.get_state()
       if stream["stream"] not in str(streams):
@@ -576,7 +651,6 @@ class Client(dbus.service.Object):
     else:
       self.w.stream_view.set_state([stream])
 
-
   @dbus.service.method("com.GwibberClient")
   def list(self):
     """
