@@ -1,9 +1,13 @@
 
-import os, json, urlparse, resources, util, sqlite3
+import os, json, urlparse, util
+import subprocess
 import gtk, gobject, pango, webkit, time, gconf
+import util
 import microblog.util
+from microblog.util import resources
 from microblog import config
 
+
 import gettext
 from gettext import lgettext as _
 if hasattr(gettext, 'bind_textdomain_codeset'):
@@ -21,6 +25,9 @@ try:
 except:
   pass
 
+error_accounts = []
+notified_errors = {}
+
 gtk.gdk.threads_init()
 
 if "gwibber" not in urlparse.uses_query:
@@ -35,6 +42,7 @@ class Model(gobject.GObject):
 
     self.settings = config.Preferences()
     self.daemon = microblog.util.getbus("Service")
+    self.messages = microblog.util.getbus("Messages")
     self.accounts = microblog.util.getbus("Accounts")
     self.streams = microblog.util.getbus("Streams")
     self.searches = microblog.util.getbus("Searches")
@@ -89,20 +97,18 @@ class Model(gobject.GObject):
     items.append({
       "name": _("Home"),
       "stream": "home",
-      "sql": "select data from messages where transient = 0",
       "account": None,
       "transient": False,
       "color": None,
     })
 
     # This list is defined just to get the strings in the template for translation
-    stream_titles = [_("Messages"), _("Replies"), _("Images"), _("Private")]
+    stream_titles = [_("Messages"), _("Replies"), _("Images"), _("Links"), _("Videos"), _("Private")]
 
-    for stream in ["messages", "replies", "images", "private"]:
+    for stream in ["messages", "replies", "images", "videos", "links", "private"]:
       items.append({
         "name": _(stream.capitalize()),
         "stream": stream,
-        "sql": "SELECT data FROM messages WHERE stream = '%s' AND transient = 0" % stream,
         "account": None,
         "transient": False,
         "color": None,
@@ -111,7 +117,6 @@ class Model(gobject.GObject):
     items.append({
       "name": _("Sent"),
       "stream": "sent",
-      "sql": "SELECT data FROM messages WHERE from_me = 1 AND transient = 0",
       "account": None,
       "transient": False,
       "color": None,
@@ -126,7 +131,6 @@ class Model(gobject.GObject):
             "name": account.get("username", "None"),
             "account": aId,
             "stream": None,
-            "sql": "SELECT data FROM messages WHERE account = '%s' AND transient = 0" % aId,
             "transient": False,
             "color": util.Color(account["color"]),
             "service": account["service"],
@@ -142,7 +146,6 @@ class Model(gobject.GObject):
               "name": _(aname.capitalize()),
               "account": aId,
               "stream": aname,
-              "sql": "SELECT data FROM messages WHERE account = '%s' AND stream = '%s' AND transient = 0" % (aId, aname),
               "transient": False,
               "color": util.Color(account["color"]),
               "service": account["service"],
@@ -152,7 +155,6 @@ class Model(gobject.GObject):
             "name": _("Sent"),
             "account": aId,
             "stream": "sent",
-            "sql": "SELECT data FROM messages WHERE account = '%s' AND from_me = 1 AND transient = 0" % aId,
             "transient": False,
             "color": util.Color(account["color"]),
             "service": account["service"],
@@ -162,10 +164,10 @@ class Model(gobject.GObject):
           tId = transient["id"]
 
           if transient["account"] == aId:
-            query = "SELECT data FROM messages WHERE transient = '%s'" % tId
-
             if transient["operation"] == "user_messages" and account["service"] in ["twitter", "identica"]:
-              query += (" OR (account = '%s' AND recipient = '%s')" % (aId, transient["parameters"]["id"]))
+              recipient = transient["parameters"]["id"]
+            else:
+              recipient = False
 
             if transient["name"] == "Lists": transient["name"] = _("Lists")
 
@@ -173,8 +175,8 @@ class Model(gobject.GObject):
               "name": transient["name"],
               "account": aId,
               "stream": self.features[transient["operation"]]["stream"],
-              "sql": query,
               "transient": tId,
+              "recipient": recipient,
               "color": util.Color(account["color"]),
               "service": account["service"],
             })
@@ -185,7 +187,6 @@ class Model(gobject.GObject):
         "name": _("Search"),
         "account": None,
         "stream": "search",
-        "sql": "SELECT data FROM messages WHERE stream = 'search'",
         "transient": False,
         "color": None,
         "items": [],
@@ -196,7 +197,6 @@ class Model(gobject.GObject):
       searches["items"].append({
         "name": search["name"],
         "account": None,
-        "sql": "SELECT data FROM messages WHERE transient = '%s'" % sId,
         "stream": "search",
         "transient": sId,
         "color": None,
@@ -253,7 +253,7 @@ class WebUi(webkit.WebView):
 
     template = open(template_path).read()
     template = Template(template, lookup=TemplateLookup(directories=lookup_paths))
-    content = template.render(theme=util.get_theme_colors(), resources=resources, _=_, **kwargs)
+    content = template.render(theme=util.get_theme_colors(), util=util, resources=resources, _=_, **kwargs)
 
     # Avoid navigation redraw crashes
     if isinstance(self, Navigation) and not self.get_property("visible"):
@@ -326,9 +326,13 @@ class SingleStreamUi(gtk.VBox):
     
     self.search_box = GwibberSearch()
     self.search_box.connect("search", self.on_search)
+
     
     layout = gtk.VBox(spacing=5)
     layout.pack_start(self.search_box, False)
+    if hasattr(gtk, "InfoBar"): 
+      self.infobox = GwibberInfoBox()
+      layout.pack_start(self.infobox, False, True, 0)
     layout.pack_start(self.messages, True)
 
     # Build the pane layout
@@ -657,8 +661,9 @@ class MessageStream(gtk.HBox):
     self.previous_stream = None
     self.messages = None
 
-    self.add_events(gtk.gdk.SCROLL_MASK)
+    self.add_events(gtk.gdk.KEY_PRESS_MASK | gtk.gdk.SCROLL_MASK)
     self.connect("scroll-event", self.on_viewport_scroll)
+    self.connect("key-press-event", self.on_viewport_key)
 
     self.message_view = MessageStreamView(model)
     self.message_view.connect("load-finished", self.on_scroll)
@@ -676,6 +681,25 @@ class MessageStream(gtk.HBox):
     self.pack_start(self.message_scroll, True, True)
     self.pack_start(self.scrollbar, False)
 
+  def on_viewport_key(self, widget, event):
+    if not event.state & gtk.gdk.SHIFT_MASK: return
+    if 'Down' == gtk.gdk.keyval_name(event.keyval):
+        value = int(self.scrollbar.get_value()) + 1
+        self.scrollbar.set_value(value)
+    elif 'Page_Down' == gtk.gdk.keyval_name(event.keyval):
+        value = int(self.scrollbar.get_value()) + 3
+        self.scrollbar.set_value(value)
+    elif 'Up' == gtk.gdk.keyval_name(event.keyval):
+        value = int(self.scrollbar.get_value()) - 1
+        self.scrollbar.set_value(value)
+    elif 'Page_Up' == gtk.gdk.keyval_name(event.keyval):
+        value = int(self.scrollbar.get_value()) - 3
+        self.scrollbar.set_value(value)
+    elif 'Home' == gtk.gdk.keyval_name(event.keyval):
+        self.scrollbar.set_value(0)
+    elif 'End' == gtk.gdk.keyval_name(event.keyval):
+        self.scrollbar.set_value(100)
+        
   def on_viewport_scroll(self, widget, event):
     value = int(self.scrollbar.get_value())
 
@@ -683,12 +707,15 @@ class MessageStream(gtk.HBox):
     elif event.direction == gtk.gdk.SCROLL_UP: value -= 1
 
     self.scrollbar.set_value(value)
-    self.handle_scroll(value)
 
   def on_scroll(self, *args):
     self.handle_scroll(int(self.scrollbar.get_value()))
 
   def handle_scroll(self, value):
+    if value > len(self.messages):
+      value = len(self.messages)
+    elif value < 0:
+      value = 0
     self.message_view.execute_script("""
     document.location.hash = "";
     document.location.hash = "msg-%s"
@@ -736,12 +763,16 @@ class MessageStreamView(WebUi):
     accounts = dict((a["id"], a) for a in accounts)
     messages = []
     seen = {}
+    time = 0
+    orderby = "time"
+    order = "desc"
+    limit = 50
 
-    if len(streams) == 1 and "sql" in streams[0]:
-      with sqlite3.connect(SQLITE_DB_FILENAME) as db:
-        data = db.execute("%s order by time desc limit %s" % (streams[0]["sql"], count)).fetchall()
-        msgs = json.loads("[%s]" % ",".join(i[0] for i in data))
-      
+    if len(streams) == 1:
+      # get message data for selected stream
+      # stream, account, time, transient, recipient, orderby, order, limit
+      msgs = json.loads(self.model.streams.Messages(streams[0]["stream"] or "all", streams[0]["account"] or "all", time, streams[0]["transient"] or "0", streams[0].has_key("recipient") and streams[0]["recipient"] or "0", orderby, order, limit))
+     
       for item in msgs:
         message = item
         message["dupes"] = []
@@ -783,6 +814,103 @@ class MessageStreamView(WebUi):
 
     return [m["txtid"] for m in messages]
 
+class GwibberInfoBox(gtk.VBox):
+  def __init__(self, error=None):
+    gtk.VBox.__init__(self, spacing=2)
+    self.service = microblog.util.getbus("Service")
+    self.service.connect_to_signal("Error", self.add_info_bar)
+    self.show()
+
+  def add_info_bar(self, data):
+    data = json.loads(data)
+    if data.has_key("error"):
+      if data["error"]["account"]["id"] in error_accounts:
+        return
+      if notified_errors.has_key(data["error"]["account"]["id"]):
+        if data["error"]["type"] in notified_errors[data["error"]["account"]["id"]]:
+          return
+        else:
+          notified_errors[data["error"]["account"]["id"]].append(data["error"]["type"])
+      print notified_errors
+      error_accounts.append(data["error"]["account"]["id"])
+      bar = GwibberInfoBar(data=data)
+      self.show()
+      bar.connect("destroy", self.on_close)
+      self.pack_end(bar, True)
+
+  def on_close(self, *args):
+    if len(self.children()) < 1:
+      self.hide()
+
+
+# FIXME    if hasattr(gtk, "InfoBar"):
+
+class GwibberInfoBar(hasattr(gtk, "InfoBar") and gtk.InfoBar or gtk.VBox):
+  def __init__(self, data=None):
+    gtk.InfoBar.__init__(self)
+    self.service = microblog.util.getbus("Service")
+    self.set_message_type(gtk.MESSAGE_ERROR)
+    #self.set_no_show_all(True)
+
+    self.account = data["error"]["account"]
+    content_area = self.get_content_area()
+    
+    #if not account.has_key("service"):
+    #  icon = gtk.image_new_from_file(resources.get_ui_asset("gwibber.png"))
+    #else:
+    icon = gtk.image_new_from_file(resources.get_ui_asset("icons/breakdance/16x16/%s.png" % data["error"]["account"]["service"]))
+    
+    label = gtk.Label(data["error"]["message"])
+
+    label.set_use_markup(True)
+    label.set_ellipsize(pango.ELLIPSIZE_END)
+
+    content_area.pack_start(icon, False, False, 0)
+    content_area.pack_start(label, True, True, 0)
+    self.pack_start(content_area, False)
+
+    close_button = gtk.Button()
+    close_button.set_image(gtk.image_new_from_stock (gtk.STOCK_CLOSE, gtk.ICON_SIZE_BUTTON))
+    close_button.set_tooltip_text(_("Close"))
+    close_button.connect("clicked", self.on_error_close_clicked_cb)
+    self.pack_end(close_button, False)
+
+    if data["error"]["type"] == "auth":
+      edit_button = gtk.Button()
+      edit_button.set_image(gtk.image_new_from_stock (gtk.STOCK_EDIT, gtk.ICON_SIZE_BUTTON))
+      edit_button.set_tooltip_text(_("Edit Account"))
+      edit_button.connect("clicked", self.on_error_edit_clicked_cb, data["error"]["account"]["id"], data["error"]["message"])
+      self.pack_end(edit_button, False)
+
+    retry_button = gtk.Button()
+    retry_button.set_image(gtk.image_new_from_stock (gtk.STOCK_REFRESH, gtk.ICON_SIZE_BUTTON))
+    retry_button.set_tooltip_text(_("Retry"))
+    retry_button.connect("clicked", self.on_error_retry_clicked_cb, data["error"]["account"]["id"])
+    self.pack_end(retry_button, False)
+
+    self.show_all()
+
+  def on_error_edit_clicked_cb(self, w, account, message, *args):
+    util.show_accounts_dialog(account, message, self, self.clear)
+
+  def on_error_close_clicked_cb(self, *args):
+    self.clear()
+
+  def on_error_retry_clicked_cb(self, w, account, *args):
+    self.service.PerformOp(json.dumps({
+      "account": account,
+      "operation": "receive",
+      "args": {},
+      "transient": False,
+    }))
+    self.clear()
+
+  def clear(self):
+    for child in self.children(): child.destroy()
+    self.destroy()
+    error_accounts.remove(self.account["id"])
+     
+  
 class GwibberSearch(gtk.HBox):
   __gsignals__ = {
     "search": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (str,)),
@@ -830,14 +958,16 @@ class GwibberSearch(gtk.HBox):
 class Input(gtk.Frame):
   __gsignals__ = {
     "submit": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (str, int)),
-    "changed": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (str, int))
+    "changed": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (str, int)),
+    "clear": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, ())
   }
 
-  def __init__(self):
+  def __init__(self, content=None):
     gtk.Frame.__init__(self)
 
-    self.textview = InputTextView()
+    self.textview = InputTextView(content=content)
     self.textview.connect("submit", self.do_submit_event)
+    self.textview.connect("clear", self.do_clear_event)
     self.textview.get_buffer().connect("changed", self.do_changed_event)
 
     scroll = gtk.ScrolledWindow()
@@ -857,6 +987,9 @@ class Input(gtk.Frame):
   def clear(self):
     self.set_text("")
 
+  def do_clear_event(self, seq):
+    self.emit("clear")
+
   def do_changed_event(self, tb):
     text = self.textview.get_text()
     chars = self.textview.get_char_count()
@@ -869,14 +1002,17 @@ class Input(gtk.Frame):
 
 class InputTextView(gtk.TextView):
   __gsignals__ = {
-    "submit": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, ())
+    "submit": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, ()),
+    "clear": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, ())
   }
 
-  def __init__(self):
+  def __init__(self, content=None):
     gtk.TextView.__init__(self)
     self.drawable = None
     self.model = Model()
 
+    self.content = content
+
     self.overlay_color = util.get_theme_colors()["text"].darker(3).hex
     self.overlay_text = '<span weight="bold" size="xx-large" foreground="%s">%s</span>'
 
@@ -894,7 +1030,7 @@ class InputTextView(gtk.TextView):
     # set properties
     self.set_border_width(0)
     self.set_accepts_tab(True)
-    self.set_editable(True)
+    self.set_editable(False)
     self.set_cursor_visible(True)
     self.set_wrap_mode(gtk.WRAP_WORD_CHAR)
     self.set_left_margin(2)
@@ -971,6 +1107,10 @@ class InputTextView(gtk.TextView):
     ww, wh = self.drawable.get_size()
     tw, th = self.pango_overlay.get_pixel_size()
     self.drawable.draw_layout(gc, ww - tw - 2, wh - th, self.pango_overlay)
+    self.set_editable(True)
+    if self.content:
+      self.get_buffer().set_text(self.content)
+      self.content = None
 
   def on_text_changed(self, w):
     chars = self.get_char_count()
@@ -985,3 +1125,4 @@ class InputTextView(gtk.TextView):
 
 gtk.binding_entry_add_signal(InputTextView, gtk.keysyms.Return, 0, "submit")
 gtk.binding_entry_add_signal(InputTextView, gtk.keysyms.KP_Enter, 0, "submit")
+gtk.binding_entry_add_signal(InputTextView, gtk.keysyms.Escape, 0, "clear")
