@@ -28,7 +28,8 @@ try:
 except:
   pass
 
-import gwibber.gwui, gwibber.microblog.util, gwibber.resources
+import gwibber.gwui, gwibber.microblog.util
+
 import gettext
 from gettext import lgettext as _
 if hasattr(gettext, 'bind_textdomain_codeset'):
@@ -36,14 +37,14 @@ if hasattr(gettext, 'bind_textdomain_codeset'):
 gettext.textdomain('gwibber')
 
 class GwibberPosterVBox(gtk.VBox):
-  def __init__(self):
+  def __init__(self, content=None):
     gtk.VBox.__init__(self)
     DBusGMainLoop(set_as_default=True)
     loop = gobject.MainLoop()
     self.service = gwibber.microblog.util.getbus("Service")
     self.stream_model = gwibber.gwui.Model()
 
-    self.input = gwibber.gwui.Input()
+    self.input = gwibber.gwui.Input(content=content)
     self.input.connect("submit", self.on_input_activate)
     self.input.connect("changed", self.on_input_changed)
     self.input_splitter = gtk.VPaned()
