@@ -34,7 +34,7 @@ class Message:
   def __init__(self, client, data):
     self.client = client
     self.account = client.account
-    self.protocol = client.account["protocol"]
+    self.service = client.account["service"]
     self.username = client.account["username"]
 
     self.sender = data["creator"]["fullname"]
@@ -72,7 +72,7 @@ class Comment:
   def __init__(self, client, data):
     self.client = client
     self.account = client.account
-    self.protocol = client.account["protocol"]
+    self.service = client.account["service"]
     self.username = client.account["username"]
 
     self.sender = data["user"]["fullname"]
@@ -97,7 +97,7 @@ class FriendPosition:
   def __init__(self, client, data):
     self.client = client
     self.account = client.account
-    self.protocol = client.account["protocol"]
+    self.service = client.account["service"]
     self.username = client.account["username"]
     self.sender = data["fullname"]
     self.sender_nick = data["login"]
