@@ -3,15 +3,22 @@
 
 import multiprocessing, threading, traceback, json
 import gobject, dbus, dbus.service
-import twitter, identica, statusnet, flickr, facebook
-import qaiku, friendfeed, digg, buzz, pingfm
 import sqlite3, mx.DateTime, re, uuid
 import urlshorter, storage, network, util, config
+from gettext import lgettext as _
+import signal
 
 from util import log
 from util import resources
 from util import exceptions
 from util.const import *
+import subprocess
+
+try:
+  from gi.repository import Unity, Dbusmenu
+except:
+  Unity = None
+  Dbusmenu = None
 
 # Try to import * from custom, install custom.py to include packaging 
 # customizations like distro API keys, etc
@@ -29,49 +36,29 @@ gobject.threads_init()
 
 log.logger.name = "Gwibber Dispatcher"
 
-PROTOCOLS = {
-  "twitter": twitter,
-  "identica": identica,
-  "flickr": flickr,
-  "facebook": facebook,
-  "friendfeed": friendfeed,
-  "statusnet": statusnet,
-  "twitter": twitter,
-  "digg": digg,
-  "qaiku": qaiku,
-  "buzz": buzz,
-  "pingfm": pingfm,
-}
+# Dynamically build a list of available service plugins
+PROTOCOLS = {}
+for p in util.resources.get_plugin_dirs()[0]:
+    PROTOCOLS[str(p)] = __import__("%s" % p, fromlist='*')
+    print "Loading plugin %s version %s" % (PROTOCOLS[str(p)].PROTOCOL_INFO["name"], PROTOCOLS[str(p)].PROTOCOL_INFO["version"])
+    #print "Path %s" % str(PROTOCOLS[str(p)].__file__)
+    # FIXME: Figure out why the logger doesn't log here
+    #log.logger.info("Loading plugin for %s", p)
 
 FEATURES = json.loads(GWIBBER_OPERATIONS)
 SERVICES = dict([(k, v.PROTOCOL_INFO) for k, v in PROTOCOLS.items()])
 SETTINGS = config.Preferences()
 
+if SETTINGS["interval"] < 5:
+  #log.logger.info( "Migrating refresh interval from %d to %d", SETTINGS["interval"], 5)
+  SETTINGS["interval"] = 5
+
+
 def perform_operation((account, opname, args, transient)):
   try:
     stream = FEATURES[opname]["stream"] or opname
     logtext = "<%s:%s>" % (account["service"], opname)
 
-    """ Dead code from merge. Do we still need this?
-    if not 'failed_id' in globals():
-      global failed_id
-      failed_id = []
-
-    for key, val in account.items():
-      if isinstance(val, str) and val.startswith(":KEYRING:"):
-        value = util.keyring.get_account_password(account["_id"])
-
-        if value is None:
-          if account["_id"] not in failed_id:
-            log.logger.debug("Adding %s to failed_id global", account["_id"])
-            failed_id.append(account["_id"])
-            log.logger.debug("Raising error to resolve failure for %s", account["_id"])
-            raise exceptions.GwibberServiceError("keyring")
-          return ("Failure", 0)
-
-        account[key] = value
-    """
-
     logtext = "<%s:%s>" % (account["service"], opname)
     log.logger.debug("%s Performing operation", logtext)
 
@@ -82,31 +69,44 @@ def perform_operation((account, opname, args, transient)):
 
     if message_data is not None:
       for m in message_data:
-        if isinstance(m, dict) and m.has_key("mid"):
-          m["id"] = uuid.uuid1().hex
-          m["operation"] = opname
-          m["stream"] = stream
-          m["transient"] = transient
-          m["rtl"] = util.isRTL(re.sub(text_cleaner, "", m["text"].decode('utf-8')))
+        try: 
+          if isinstance(m, dict) and m.has_key("mid"):
+            m["id"] = uuid.uuid1().hex
+            m["operation"] = opname
+            m["stream"] = stream
+            m["transient"] = transient
+            m["rtl"] = util.isRTL(re.sub(text_cleaner, "", m["text"].decode('utf-8')))
+            if m.has_key("type"):
+              if m["type"] == "link": m["stream"] = "links"
+              if m["type"] == "video": m["stream"] = "videos"
+              if m["type"] == "photo": m["stream"] = "images"
 
-          log.logger.debug("%s Adding record", logtext)
+            log.logger.debug("%s Adding record", logtext)
 
-          new_messages.insert(0, (
-            m["id"],
-            m["mid"],
-            m["account"],
-            account["service"],
-            opname,
-            transient,
-            stream,
-            m["time"],
-            m["text"],
-            m.get("sender", {}).get("is_me", None), 
-            m.get("to_me", None),
-            m.get("sender", {}).get("nick", None),
-            m.get("reply", {}).get("nick", None),
-            json.dumps(m)
-          ))
+            new_messages.insert(0, (
+              m["id"],
+              m["mid"],
+              m["account"],
+              account["service"],
+              opname,
+              transient,
+              m["stream"] or stream,
+              m["time"],
+              m["text"],
+              m.get("sender", {}).get("is_me", None), 
+              m.get("to_me", None),
+              m.get("sender", {}).get("nick", None),
+              m.get("reply", {}).get("nick", None),
+              json.dumps(m)
+            ))
+          elif isinstance(m, dict) and m.has_key("error"):
+            new_messages.insert(0, (
+              "error",
+              json.dumps(m)
+            ))
+        except Exception as e:
+          if not "logtext" in locals(): logtext = "<UNKNOWN>"
+          log.logger.error("%s Operation failed", logtext)
 
     log.logger.debug("%s Finished operation", logtext)
     return ("Success", new_messages)
@@ -159,6 +159,13 @@ class OperationCollector:
     return {}
 
   def validate_operation(self, acct, opname, enabled="receive_enabled"):
+    # if account doesn't have the required feature or is disabled, return
+    if enabled in acct:
+      if not acct[enabled]: return
+    else: 
+      return
+    # if there is an account for a service that gwibber doesn't no about, return
+    if not acct["service"] in SERVICES: return
     service = SERVICES[acct["service"]]
     return acct["service"] in PROTOCOLS and \
            opname in service["features"] and \
@@ -254,17 +261,56 @@ class Dispatcher(dbus.service.Object):
     self.streams = storage.StreamManager(self.db)
     self.messages = storage.MessageManager(self.db)
     self.collector = OperationCollector(self)
+ 
+    # Monitor the connection
+    self.connection_monitor = util.getbus("Connection")
+    self.connection_monitor.connect_to_signal("ConnectionOnline", self.on_connection_online)
+    self.connection_monitor.connect_to_signal("ConnectionOffline", self.on_connection_offline)
+
+    self.indicate = None 
 
     if indicate and util.resources.get_desktop_file():
       self.indicate = indicate.indicate_server_ref_default()
       self.indicate.set_type("message.gwibber")
       self.indicate.set_desktop_file(util.resources.get_desktop_file())
-      self.indicate.connect("server-display", self.on_indicator_activate)
+      self.indicate.connect("server-display", self.on_indicator_server_activate)
       self.indicate.connect("interest-added", self.on_indicator_interest_added)
       self.indicate.connect("interest-removed", self.on_indicator_interest_removed)
       self.indicate.show()
     self.indicator_items = {}
     self.notified_items = []
+    self.notified_errors = {}
+    self.messages_indicator = None
+    self.replies_indicator = None
+    self.private_indicator = None
+    self.unseen_counts = {}
+    for s in "messages", "replies", "private":
+      self.unseen_counts[s] = 0
+
+    if Unity and Dbusmenu:
+      launcher = Unity.LauncherEntry.get_for_desktop_id ("gwibber.desktop")
+      ql = Dbusmenu.Menuitem.new ()
+      refresh_menu = Dbusmenu.Menuitem.new ()
+      refresh_menu.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Refresh"))
+      refresh_menu.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
+      refresh_menu.connect("item-activated", self.refresh)
+      ql.child_append (refresh_menu)
+      accounts_menu = Dbusmenu.Menuitem.new ()
+      accounts_menu.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Accounts"))
+      accounts_menu.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
+      accounts_menu.connect("item-activated", self.show_accounts)
+      ql.child_append (accounts_menu)
+      preferences_menu = Dbusmenu.Menuitem.new ()
+      preferences_menu.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Preferences"))
+      preferences_menu.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
+      preferences_menu.connect("item-activated", self.show_preferences)
+      ql.child_append (preferences_menu)
+      quit_menu = Dbusmenu.Menuitem.new ()
+      quit_menu.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Quit"))
+      quit_menu.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
+      quit_menu.connect("item-activated", self.shutdown)
+      ql.child_append (quit_menu)
+      launcher.set_property("quicklist", ql)
 
     self.refresh_count = 0
     self.mainloop = loop
@@ -272,14 +318,58 @@ class Dispatcher(dbus.service.Object):
 
     self.refresh_timer_id = None
 
+    self.maintDone = False
+    self.maintRunning = False
+    self.refreshRunning = False
+
     if autorefresh:
-      self.refresh()
+      if self.refresh_timer_id:
+        gobject.source_remove(self.refresh_timer_id)
+      # wait a few seconds before alerting the world we are online
+      self.refresh_timer_id = gobject.timeout_add_seconds(int(10), self.refresh)
 
     self.accounts_service = util.getbus("Accounts")
     self.accounts_service.connect_to_signal("Updated", self.on_account_updated)
     self.accounts_service.connect_to_signal("Deleted", self.on_account_deleted)
     self.accounts_service.connect_to_signal("Created", self.on_account_created)
 
+  def show_preferences(self, *args):
+    subprocess.Popen("gwibber-preferences", shell=False)
+
+  def show_accounts(self, *args):
+    subprocess.Popen("gwibber-accounts", shell=False)
+
+  def shutdown(self, *args):
+    subprocess.Popen(["pkill", "gwibber"], shell=False)
+    self.Quit()
+
+  def do_maintenance(self, *args):
+    # perform some needed MessageManager maintenance
+    if self.maint_timer_id:
+      gobject.source_remove(self.maint_timer_id)
+    if self.refreshRunning:
+      self.maint_timer_id = gobject.timeout_add_seconds(60, self.do_maintenance)
+      return False
+
+    self.maintRunning = True
+    self.messages.maintenance()
+    self.maintRunning = False
+    self.maintDone = True
+    return False
+    
+  def on_connection_online(self, *args):
+    log.logger.info("Dispatcher Online, initiating a refresh")
+    if self.refresh_timer_id:
+      gobject.source_remove(self.refresh_timer_id)
+    # wait a few seconds before alerting the world we are online
+    self.refresh_timer_id = gobject.timeout_add_seconds(int(10), self.refresh)
+
+  def on_connection_offline(self, *args):
+    self.refreshRunning = False
+    log.logger.info("Dispatcher Offline, suspending operations")
+    if self.refresh_timer_id:
+      gobject.source_remove(self.refresh_timer_id)
+
   def on_account_updated(self, account):
     pass
 
@@ -297,10 +387,12 @@ class Dispatcher(dbus.service.Object):
       pass
 
   @dbus.service.signal("com.Gwibber.Service")
-  def LoadingComplete(self): pass
+  def LoadingComplete(self):
+    self.refreshRunning = False
 
   @dbus.service.signal("com.Gwibber.Service")
-  def LoadingStarted(self): pass
+  def LoadingStarted(self):
+    self.refreshRunning = True
 
   @dbus.service.method("com.Gwibber.Service")
   def Refresh(self):
@@ -337,6 +429,18 @@ class Dispatcher(dbus.service.Object):
     if operation:
       self.perform_async_operation(operation)
 
+  @dbus.service.method("com.Gwibber.Service", in_signature="s")
+  def UpdateIndicators(self, stream):
+    """
+    Update counts in messaging indicators
+    example:
+            import dbus
+            obj = dbus.SessionBus().get_object("com.Gwibber.Service", "/com/gwibber/Service")
+            service = dbus.Interface(obj, "com.Gwibber.Service")
+            service.UpdateIndicators("stream")
+    """
+    self.handle_indicator_counts(stream)
+
   @dbus.service.method("com.Gwibber.Service", in_signature="s")
   def SendMessage(self, message):
     """
@@ -439,7 +543,7 @@ class Dispatcher(dbus.service.Object):
             service = dbus.Interface(obj, "com.Gwibber.Service")
             res = service.IndicatorInterestCheck()
     """
-    if indicate:
+    if indicate and self.indicate:
       return self.indicate.check_interest(indicate.INTEREST_SERVER_DISPLAY)
     else:
       return False
@@ -450,6 +554,27 @@ class Dispatcher(dbus.service.Object):
   @dbus.service.signal("com.Gwibber.Service")
   def IndicatorInterestRemoved(self): pass
 
+  @dbus.service.signal("com.Gwibber.Service", signature="s")
+  def Error(self, error): pass
+
+  @dbus.service.method("com.Gwibber.Service", in_signature="s")
+  def clear_error(self, error):
+    error = json.loads(error)
+    self.notified_errors["account"]["service"] = None
+    
+  def send_error_notify(self, error):
+    error = json.loads(error)["error"]
+
+    if self.notified_errors.has_key(error["account"]["service"]):
+      if self.notified_errors[error["account"]["service"]] == error["message"]:
+        return
+    if util.can_notify:
+      icon = util.resources.get_ui_asset("icons/breakdance/scalable/%s.svg" % error["account"]["service"])
+      if not icon:
+        icon = util.resources.get_ui_asset("gwibber.svg")
+      util.notify(error["account"]["service"], error["message"], icon, 2000)
+    self.notified_errors[error["account"]["service"]] = error["message"]
+
   def perform_async_operation(self, iterable):
     t = MapAsync(perform_operation, iterable, self.loading_complete, self.loading_failed, self.workerpool)
     t.join()
@@ -458,11 +583,19 @@ class Dispatcher(dbus.service.Object):
     self.refresh_count += 1
     
     items = []
+    errors = []
     for o in output:
       for o2 in o[1]:
         if len(o2) > 1:
-          items.append(o2)
-
+          if o2[0] != "error":
+            with sqlite3.connect(SQLITE_DB_FILENAME) as db:
+              if len(db.execute("""select * from messages where mid = '%s' and account = '%s' and stream = '%s'""" % (o2[1], o2[2], o2[6])).fetchall()) > 0:
+                self.messages.Message("update", o2[-1])
+              else:
+                self.messages.Message("new", o2[-1])
+            items.append(o2)
+          else:
+            errors.append(o2)
     with sqlite3.connect(SQLITE_DB_FILENAME) as db:
       oldid = db.execute("select max(ROWID) from messages").fetchone()[0] or 0
       
@@ -470,25 +603,75 @@ class Dispatcher(dbus.service.Object):
             ",".join(self.messages.columns),
             ",".join("?" * len(self.messages.columns))), items)
 
+      for s in "messages", "replies", "private":
+        if s == "messages": 
+          to_me = 0 
+        else: to_me = 1
+        count = db.execute("SELECT count(mid) FROM messages WHERE stream = ? AND from_me = 0 and to_me = ? AND ROWID > ?", (s,to_me,oldid)).fetchone()[0] or 0
+        self.unseen_counts[s] = self.unseen_counts[s] + int(count)
+  
+      self.update_indicators(self.unseen_counts)
+
       new_items = db.execute("""
         select * from (select * from messages where operation == "receive" and ROWID > %s and to_me = 0 ORDER BY time DESC LIMIT 10) as a union
-        select * from (select * from messages where operation IN ("receive", "private") and ROWID > %s and to_me != 0 ORDER BY time DESC LIMIT 10) as b 
+        select * from (select * from messages where operation IN ("receive","private") and ROWID > %s and to_me != 0 ORDER BY time DESC LIMIT 10) as b
         ORDER BY time ASC""" % (oldid, oldid)).fetchall()
 
-
       for i in new_items:
           self.new_message(i)
     
+    for error in errors:
+      self.Error(error[1])
+      self.send_error_notify(error[1])
+
     self.LoadingComplete()
     log.logger.info("Loading complete: %s - %s", self.refresh_count, [o[0] for o in output])
 
+  def update_indicators(self, counts):
+    if indicate:
+      if counts.has_key("private"):
+        if not self.private_indicator:
+          self.private_indicator = indicate.Indicator() if hasattr(indicate, "Indicator") else indicate.IndicatorMessage()
+          self.private_indicator.connect("user-display", self.on_indicator_activate)
+          self.private_indicator.set_property("name", _("Private"))
+          self.private_indicator.set_property("stream", "private")
+          self.private_indicator.show()
+        self.private_indicator.set_property("count", str(counts["private"]))
+        if counts["private"] > 0:
+          self.private_indicator.set_property_bool("draw-attention", True)
+        if self.private_indicator not in self.indicator_items:
+          self.indicator_items["private"] = self.private_indicator
+        log.logger.debug("Private Messages Indicator count updated to %s", counts["private"])
+      if counts.has_key("replies"):
+        if not self.replies_indicator:
+          self.replies_indicator = indicate.Indicator() if hasattr(indicate, "Indicator") else indicate.IndicatorMessage()
+          self.replies_indicator.connect("user-display", self.on_indicator_activate)
+          self.replies_indicator.set_property("name", _("Replies"))
+          self.replies_indicator.set_property("stream", "replies")
+          self.replies_indicator.show()
+        self.replies_indicator.set_property("count", str(counts["replies"]))
+        if self.replies_indicator not in self.indicator_items:
+          self.indicator_items["replies"] = self.replies_indicator
+        log.logger.debug("Replies Indicator count updated to %s", counts["replies"])
+      if counts.has_key("messages"):
+        if not self.messages_indicator:
+          self.messages_indicator = indicate.Indicator() if hasattr(indicate, "Indicator") else indicate.IndicatorMessage()
+          self.messages_indicator.connect("user-display", self.on_indicator_activate)
+          self.messages_indicator.set_property("name", _("Messages"))
+          self.messages_indicator.set_property("stream", "messages")
+          self.messages_indicator.show()
+        self.messages_indicator.set_property("count", str(counts["messages"]))
+        if self.messages_indicator not in self.indicator_items:
+          self.indicator_items["messages"] = self.messages_indicator
+        log.logger.debug("Messages Indicator count updated to %s", counts["messages"])
+
   def on_indicator_interest_added(self, server, interest):
     self.IndicatorInterestAdded()
 
   def on_indicator_interest_removed(self, server, interest):
     self.IndicatorInterestRemoved()
 
-  def on_indicator_activate(self, indicator, timestamp):
+  def on_indicator_server_activate(self, indicator, timestamp=None):
     dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
     client_bus = dbus.SessionBus()
     log.logger.debug("Raising gwibber client")
@@ -502,22 +685,22 @@ class Dispatcher(dbus.service.Object):
     except dbus.DBusException:
       log.logger.error("Indicator activate failed:\n%s", traceback.format_exc())
 
-
-  def on_indicator_reply_activate(self, indicator, timestamp):
-    log.logger.debug("Raising gwibber client, focusing replies stream")
+  def on_indicator_activate(self, indicator, timestamp=None):
+    if not indicate: return
+    stream = indicator.get_property("stream")
+    log.logger.debug("Raising gwibber client, focusing %s stream", stream)
+    try:
+      self.handle_indicator_counts(stream)
+    except:
+      pass
     client_bus = dbus.SessionBus()
     try:
       client_obj = client_bus.get_object("com.GwibberClient", "/com/GwibberClient")
       gw = dbus.Interface(client_obj, "com.GwibberClient")
-      gw.show_replies(reply_handler=self.handle_focus_reply,
+      gw.show_stream(stream, reply_handler=self.handle_focus_reply,
                       error_handler=self.handle_focus_error)
-      for i in self.indicator_items.values():
-        try:
-          i.hide()
-        except:
-          pass
     except dbus.DBusException:
-      log.logger.error("Indicator reply activate failed:\n%s", traceback.format_exc())
+      log.logger.error("Indicator activation failed:\n%s", traceback.format_exc())
 
   def handle_focus_reply(self, *args):
     log.logger.debug("Gwibber Client raised")
@@ -525,16 +708,20 @@ class Dispatcher(dbus.service.Object):
   def handle_focus_error(self, *args):
     log.logger.error("Failed to raise client %s", args)
 
+  def handle_indicator_counts(self, stream):
+    if not indicate: return
+    if self.indicator_items.has_key(stream):
+      self.indicator_items[stream].set_property("count", str(0))
+      if stream == "private":
+        self.private_indicator.set_property_bool("draw-attention", False)
+    self.unseen_counts[stream] = 0
+
   def new_message(self, data):
     message = json.loads(data[-1])
     if message["transient"]:
       log.logger.debug("Message %s is transient, not notifying", message["id"])
       return 
 
-    if indicate and message["to_me"]:
-      if not self.indicator_items.has_key(str(message["id"])):
-        gobject.idle_add(self.handle_indicator_item, message)
-
     if util.can_notify and  str(message["mid"]) not in self.notified_items:
       self.notified_items.append(message["mid"])
       if SETTINGS["notify_mentions_only"] and message["to_me"]: 
@@ -550,28 +737,27 @@ class Dispatcher(dbus.service.Object):
     else:
       sender_name = message["sender"].get("nick", message["sender"].get("name", ""))
 
+    notify_text = ""
+    if len(message["text"]) > 0: 
+      notify_text = message["text"]
+    elif message.has_key("stream"):
+      if message["stream"] == "images":
+        notify_text = _("has shared a photo")
+      if message["stream"] == "links":
+        notify_text = _("has shared a link")
+      if message["stream"] == "videos":
+        notify_text = _("has shared a video")
     #image = util.resources.get_ui_asset("icons/breakdance/scalable/%s.svg" % message["service"])
-    image = util.resources.get_avatar_path(message["sender"]["image"])
+    if message["sender"].has_key("image"):
+      image = util.resources.get_avatar_path(message["sender"]["image"])
+    else:
+      image = util.resources.get_ui_asset("icons/breakdance/scalable/%s.svg" % message["service"])
     if not image:
       image = util.resources.get_ui_asset("icons/breakdance/scalable/%s.svg" % message["service"])
-    util.notify(sender_name, message["text"], image, 2000)
+    util.notify(sender_name, notify_text, image, 2000)
 
     return False
 
-  def handle_indicator_item(self, message):
-    if not self.indicator_items.has_key(str(message["mid"])):
-      indicator = indicate.Indicator() if hasattr(indicate, "Indicator") else indicate.IndicatorMessage()
-      indicator.connect("user-display", self.on_indicator_reply_activate)
-      indicator.set_property("subtype", "im.gwibber")
-      indicator.set_property("sender", message["sender"].get("name", ""))
-      indicator.set_property("body", message["text"])
-      indicator.set_property_time("time",
-          mx.DateTime.DateTimeFromTicks(message["time"]).localtime().ticks())
-      indicator.show()
-      self.indicator_items[message["mid"]] = indicator
-      log.logger.debug("Message from %s added to indicator", message["sender"].get("name", ""))
-    return False
-
   def loading_failed(self, exception, tb):
     self.LoadingComplete()
     log.logger.error("Loading failed: %s - %s", exception, tb)
@@ -583,23 +769,31 @@ class Dispatcher(dbus.service.Object):
       log.logger.debug("*** Sending Message ***")
       self.perform_async_operation(operations)
 
-  def refresh(self):
+  def refresh(self, *args):
     if self.refresh_timer_id:
       gobject.source_remove(self.refresh_timer_id)
-    log.logger.debug("Refresh interval is set to %s", SETTINGS["interval"])
-    operations = []
-    
-    for o in self.collector.get_operations():
-      interval = FEATURES[o[1]].get("interval", 1)
-      if self.refresh_count % interval == 0:
-        operations.append(o)
-    
-    if operations:
-      log.logger.debug("** Starting Refresh - %s **", mx.DateTime.now())
-      self.LoadingStarted()
-      self.perform_async_operation(operations)
 
-    self.refresh_timer_id = gobject.timeout_add_seconds(int(60 * SETTINGS["interval"]), self.refresh)
+    if not self.maintRunning and not self.refreshRunning:
+      log.logger.debug("Refresh interval is set to %s", SETTINGS["interval"])
+      operations = []
+    
+      for o in self.collector.get_operations():
+        interval = FEATURES[o[1]].get("interval", 1)
+        if self.refresh_count % interval == 0:
+          operations.append(o)
+    
+      if operations:
+        log.logger.debug("** Starting Refresh - %s **", mx.DateTime.now())
+        self.LoadingStarted()
+        self.perform_async_operation(operations)
+
+      self.refresh_timer_id = gobject.timeout_add_seconds(int(60 * SETTINGS["interval"]), self.refresh)
+    else:
+      self.refresh_timer_id = gobject.timeout_add_seconds(int(30), self.refresh)
+
+    if not self.maintDone:
+      self.maint_timer_id = gobject.timeout_add_seconds(60, self.do_maintenance)
+
     return False
 
 class ConnectionMonitor(dbus.service.Object):
@@ -611,17 +805,30 @@ class ConnectionMonitor(dbus.service.Object):
     dbus.service.Object.__init__(self, bus_name, self.__dbus_object_path__)
 
     self.sysbus = dbus.SystemBus()
+
+    self.has_nm = None
+
     try:
       self.nm = self.sysbus.get_object(NM_DBUS_SERVICE, NM_DBUS_OBJECT_PATH)
       self.nm.connect_to_signal("StateChanged", self.on_connection_changed)
+      self.has_nm = True
     except:
       pass
 
   def on_connection_changed(self, state):
+    log.logger.debug("Network state changed, new state is %d", state)
+    """
+    NM_STATE_UNKNOWN = 0
+    NM_STATE_ASLEEP = 1
+    NM_STATE_CONNECTING = 2
+    NM_STATE_CONNECTED = 3
+    NM_STATE_DISCONNECTED = 4
+    """
+
     if state == NM_STATE_CONNECTED:
       log.logger.info("Network state changed to Online")
       self.ConnectionOnline()
-    if state == NM_STATE_DISCONNECTED:
+    else:
       log.logger.info("Network state changed to Offline")
       self.ConnectionOffline()
 
@@ -633,10 +840,14 @@ class ConnectionMonitor(dbus.service.Object):
 
   @dbus.service.method("com.Gwibber.Connection")
   def isConnected(self):
+    if not self.has_nm: 
+      log.logger.info("Can't determine network state, assuming online")
+      return True
     try:
       if self.nm.state() == NM_STATE_CONNECTED:
         return True
-      return False
+      else:
+        return False
     except:
       return True
 
@@ -687,7 +898,7 @@ class Translate(dbus.service.Object):
     url = "http://ajax.googleapis.com/ajax/services/language/translate"
     params = {"v": "1.0", "q": text, "langpair": "|".join((srclang, dstlang))}
     data = network.Download(url, params).get_json()
-    
+
     if data["responseStatus"] == 200:
       return data.get("responseData", {}).get("translatedText", "")
     else: return ""
