@@ -1,10 +1,15 @@
 #!/usr/bin/env python
 
 import urllib, pycurl, json, StringIO
-import traceback
 from util import log
-from util import exceptions
-from util.const import proxy_factory
+
+try:
+  import libproxy
+except:
+  libproxy = None
+
+# Completely disable libproxy support for now, it causes crashes on amd64
+libproxy = None
 
 class CurlDownloader:
   def __init__(self, url, params=None, post=False, username=None, password=None, header=None, body=None):
@@ -37,7 +42,8 @@ class CurlDownloader:
     self.content = StringIO.StringIO()
     self.curl.setopt(pycurl.WRITEFUNCTION, self.content.write)
     
-    if proxy_factory:
+    if libproxy:
+      proxy_factory = libproxy.ProxyFactory()
       log.logger.debug("libproxy: getting proxies")
       proxylist = proxy_factory.getProxies(str(url))
 
@@ -51,16 +57,15 @@ class CurlDownloader:
 
     try:
       self.curl.perform()
-    except:
-      traceback.print_exc()
-      log.logger.error("Failed to communicate with %s", str(url))
-  
+    except pycurl.error, e:
+      log.logger.error("Network failure - error: %d - %s", e[0], e[1])
+
   def get_json(self):
     try:
       return json.loads(self.get_string())
     except ValueError as e:
-      log.logger.error("Failed to parse the response, error was: %s", str(e))
-      return str(e)
+      log.logger.debug("Failed to parse the response, error was: %s", str(e))
+      return None
 
   def get_string(self):
     return self.content.getvalue()
