@@ -22,8 +22,8 @@ def parsetime(t):
 URL_SCHEMES = ('http', 'https', 'ftp', 'mailto', 'news', 'gopher',
                'nntp', 'telnet', 'wais', 'prospero', 'aim', 'webcal')
 
-URL_FORMAT = (r'(?<!\w)((?:%s):' # protocol + :
-    '/*(?!/)(?:' # get any starting /'s
+URL_FORMAT = (r'(?<!\w)(((?:%s)://|(?:www\.))' # protocol + :// or www.
+    '(?!/)(?:' # get any starting /'s
     '[\w$\+\*@&=\-/]' # reserved | unreserved
     '|%%[a-fA-F0-9]{2}' # escape
     '|[\?\.:\(\),;!\'\~](?!(?:\s|$))' # punctuation
@@ -42,7 +42,9 @@ def strip_urls(text):
 def linkify(text, subs=[], escape=True):
   if escape: text = cgi.escape(text)
   for f, r in subs: text = f.sub(r, text)
-  return PARSE_LINK.sub('<a href="\\1">\\1</a>', text)
+  link = PARSE_LINK.sub('<a href="\\1">\\1</a>', text)
+  # if link has www only, add http:// to the href
+  return re.compile(r'"www.', re.U).sub('"http://www.', link)
 
 def imgpreview(text):
   thumbre = {
