@@ -1,11 +1,6 @@
 import xdg.BaseDirectory
 from os.path import join, isdir, realpath
-from os import mkdir
-
-from os.path import join, isdir, realpath
-from os import mkdir
-
-import xdg.BaseDirectory
+from os import mkdir, environ
 
 CACHE_DIR = realpath(join(xdg.BaseDirectory.xdg_cache_home, "gwibber"))
 if not isdir(CACHE_DIR):
@@ -30,6 +25,10 @@ SQLITE_DB_FILENAME = "%s/gwibber.sqlite" % config_dir
 if not isdir(cache_dir): mkdir(cache_dir)
 if not isdir(config_dir): mkdir(config_dir)
 
+GWIBBER_TEST_DUMP = None
+if environ.has_key("GWIBBER_TEST_DUMP"):
+  GWIBBER_TEST_DUMP = environ["GWIBBER_TEST_DUMP"]
+
 DEFAULT_SETTINGS = {
   "interval": 15,
   "view": "SingleStreamUi",
@@ -63,7 +62,7 @@ RETWEET_FORMATS = {
   "service": "{R} @{nick}: {text}",
 }
 
-VERSION_NUMBER = "2.32.2"
+VERSION_NUMBER = "3.0.0.1"
 GCONF_CLIENT_DIR = "/apps/gwibber/client/"
 
 BUG_URL = "https://bugs.launchpad.net/gwibber/+filebug"
@@ -204,6 +203,30 @@ GWIBBER_OPERATIONS = """
     "transient": false 
   },
 
+  "links": {
+    "account_tree": true,
+    "dynamic": false,
+    "enabled": "receive",
+    "first_only": false,
+    "function": null,
+    "return_value": true,
+    "search": false,
+    "stream": "links",
+    "transient": false 
+  },
+
+  "videos": {
+    "account_tree": true,
+    "dynamic": false,
+    "enabled": "receive",
+    "first_only": false,
+    "function": null,
+    "return_value": true,
+    "search": false,
+    "stream": "videos",
+    "transient": false 
+  },
+
   "reply": {
     "account_tree": false,
     "dynamic": false,
@@ -337,9 +360,3 @@ GWIBBER_OPERATIONS = """
   }
 }
 """
-
-try:
-  import libproxy
-  proxy_factory = libproxy.ProxyFactory()
-except:
-  proxy_factory = None
