@@ -10,7 +10,7 @@ def get_secret(id):
     value = gnomekeyring.find_items_sync(
         gnomekeyring.ITEM_GENERIC_SECRET,
         {"id": str(id)})[0].secret
-    #mlock(value)
+    mlock(value)
   except gnomekeyring.NoMatchError:
     print id
     raise exceptions.GwibberServiceError("keyring")
