@@ -1,6 +1,7 @@
 from os.path import join, isdir, realpath
 from os import makedirs
-import logging, logging.handlers
+import logging
+import logging.handlers
 
 import xdg.BaseDirectory
 
