@@ -26,7 +26,8 @@ except:
   exit
 
 import gtk, gconf
-from gwibber import resources, util
+from gwibber import util
+from microblog.util import resources
 from microblog import config
 
 import gettext
